@@ -57,10 +57,6 @@ def register(filename, logs_location, api_url, api_key=None, dry_run=False):
     init_kwargs = {}
     if logs_location:
         init_kwargs["logs_location"] = logs_location
-    if filename:
-        init_kwargs["filename"] = filename
-    if api_url:
-        init_kwargs["api_url"] = api_url
     if api_key:
         init_kwargs["api_key"] = api_key
     gantry.init(**init_kwargs)
