@@ -1,50 +1,45 @@
 import datetime
 import functools
 import inspect
+import json
 import logging
 import math
 import os
 import random
 import sys
+import uuid
+from dataclasses import asdict
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Tuple, Union, no_type_check
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 from warnings import warn
 
-import colorama
 import pandas as pd
 import requests
 from tqdm import tqdm
 from typeguard import typechecked
 
-from gantry.config import Config
-from gantry.exceptions import (
-    GantryBatchCreationException,
-    GantryException,
-    GantryLoggingException,
-)
-from gantry.logger.constants import CHUNK_SIZE, BatchTypes, Delimiter, UploadFileType
+from gantry.exceptions import GantryBatchCreationException, GantryLoggingException
+from gantry.logger.constants import CHUNK_SIZE, BatchType
 from gantry.logger.event_builder import (
-    _build_batch_meta,
+    _build_data_link,
     _build_feedback_event,
     _build_prediction_and_feedback_events,
     _build_prediction_event,
     _build_prediction_events,
     _create_timestamp_idx,
-    _enrich_events_with_batch_id,
 )
 from gantry.logger.stores import APILogStore
+from gantry.logger.types import DataLink
 from gantry.logger.utils import (
     _batch_fail_msg,
     _batch_success_msg,
+    _build_batch_iterator,
     _check_sample_rate,
     _is_empty,
     _is_not_empty,
     _log_exception,
-    get_file_linecount,
 )
-from gantry.query.core.utils import check_response
-from gantry.serializers import serializable_value
-from gantry.utils import format_msg_with_color
+from gantry.serializers import EventEncoder, serializable_value
 
 logger = logging.getLogger(__name__)
 
@@ -72,44 +67,6 @@ class Gantry:
         self.environment = environment
         self.setup_logger(level=logging_level)
 
-    @no_type_check
-    @classmethod
-    def from_config(cls, config: Config):
-        env = config.get("environment")
-        logging_level = config.get("logging_level", "INFO")
-
-        logs_location = config.get("logs_location") or ""
-        log_location_msg = None
-
-        # TODO: use urlparse to grab the scheme instead
-        if not (logs_location.startswith("http://") or logs_location.startswith("https://")):
-            logger.error("Currently only HTTP logs location supported")
-            raise ValueError(
-                "logger was not initialized with http endpoint. "
-                "Only HTTP logs locations supported."
-            )
-
-        api_key = config.get("api_key")
-        bypass_firehose = config.get("bypass_firehose", False)
-        send_in_background = config.get("send_in_background", True)
-
-        log_store = APILogStore(
-            logs_location,
-            api_key=api_key,
-            bypass_firehose=bypass_firehose,
-            send_in_background=send_in_background,
-        )
-        log_location_msg = "Using API for logging at URI %s"
-
-        # Case when `logs_location=None`, we already output warning in red with special message,
-        # so we don't want to output another warning
-        log_location_formatted_msg = format_msg_with_color(
-            log_location_msg, colorama.Fore.BLUE, logger
-        )
-        logger.warning(log_location_formatted_msg, logs_location)
-
-        return cls(log_store, env, logging_level)
-
     def instrument(
         self,
         application: str,
@@ -202,84 +159,68 @@ class Gantry:
         inputs: List[str] = [],
         outputs: List[str] = [],
         feedbacks: List[str] = [],
-        metadata: List[str] = [],
+        tags: List[str] = [],
         feedback_id: List[str] = [],
         feedback_keys: Optional[List[str]] = None,
     ):
-        # Get info for tracking batch during processing.
-        num_events = get_file_linecount(filepath) - 1  # Subtract the header line.
-        logger.info(f"Processing {num_events} lines.")
-
-        # Read the headers of the file, infer and build linking datastruct.
-        gantry_data_link: Dict[str, Any] = {
-            "file_type": UploadFileType.CSV_WITH_HEADERS.value,
-            "num_events": num_events,
-            "application": application,
-            "version": version,
-            "log_timestamp": datetime.datetime.utcnow().isoformat(),
-            "timestamp": dict(),
-            "inputs": dict(),
-            "outputs": dict(),
-            "feedback": dict(),
-            "tags": {
-                "env": {"val": self.environment}
-            },  # This makes env.val exactly passed in rather than reference to a file column.
-            "feedback_id": dict(),
-            "feedback_keys": list(),
-        }
+        data_link = _build_data_link(
+            application,
+            self.environment,
+            version,
+            timestamp,
+            inputs,
+            outputs,
+            feedbacks,
+            tags,
+            feedback_id,
+            feedback_keys,
+            filepath=filepath,
+            batch_type=BatchType.RECORD,  # TODO: Determine this by the data_link request.
+        )
+        object_size = os.path.getsize(filepath)  # in bytes
+        path = Path(filepath)
         with open(filepath, "r") as f_in:
-            headers_line = f_in.readline().split(Delimiter.COMMA.value)
-        for col_index, raw_header in enumerate(headers_line):
-            header = raw_header.strip()
-            if header == timestamp or (not timestamp and header.startswith("timestamp")):
-                gantry_data_link["timestamp"][header] = {"ref": col_index}
-            elif header in outputs or header.startswith("output"):
-                gantry_data_link["outputs"][header] = {"ref": col_index}
-            elif header in feedbacks or header.startswith("feedback"):
-                gantry_data_link["feedback"][header] = {"ref": col_index}
-            elif header in metadata or header.startswith("metadata"):
-                gantry_data_link["tags"][header] = {"ref": col_index}
-            elif len(inputs) == 0 or header in inputs or header.startswith("input"):
-                gantry_data_link["inputs"][header] = {"ref": col_index}
-
-            if header in feedback_id or header == "feedback_id":
-                gantry_data_link["feedback_id"][header] = {"ref": col_index}
-            elif feedback_keys is not None and header in feedback_keys:
-                gantry_data_link["feedback_keys"].append(header)
-        if not gantry_data_link["tags"].get("env"):
-            raise GantryLoggingException(
-                "Please initialize gantry logger with 'environment' or specify an 'env' tag."
+            block_read = functools.partial(f_in.read, CHUNK_SIZE)
+            block_iterator = iter(block_read, "")
+            self._handle_upload(
+                block_iterator, data_link, object_size, f"{uuid.uuid4()}_{path.name}"
             )
 
+    def _handle_upload(
+        self, data_batch_iterator: Iterable, data_link: DataLink, object_size: int, filename: str
+    ):
         # Calculate part counts and generate presigned s3 urls.
-        path = Path(filepath)
-        object_size = os.path.getsize(filepath)  # in bytes
         total_num_parts = math.ceil(object_size / CHUNK_SIZE)
 
         logger.info("Initializing upload to Gantry")
         preupload_res = self.log_store._api_client.request(
             "GET",
             "/api/v1/ingest/file-preupload",
-            params={"filename": path.name, "num_parts": total_num_parts},
+            params={
+                "filename": filename,
+                "num_parts": total_num_parts,
+                "filetype": data_link.file_type,
+            },
         )
+        if "upload_urls" not in preupload_res:
+            raise GantryBatchCreationException(
+                "Failed to start batch upload. Please check your API key."
+            )
         signed_urls = preupload_res["upload_urls"]
         upload_id = preupload_res["upload_id"]
         file_key = preupload_res["key"]
 
         # Upload the parts.
         parts = []
-        with open(filepath, "r") as f_in:
-            block_read = functools.partial(f_in.read, CHUNK_SIZE)
-            block_iterator = iter(block_read, "")
-            part_num = 1  # AWS S3 needs part num to start with 1.
-            for block in tqdm(
-                block_iterator, desc="Uploading to Gantry Bucket: ", total=total_num_parts
-            ):
-                # TODO: parallelize this client side?
-                response = requests.put(signed_urls[part_num - 1], data=block)
-                etag = str(response.headers["ETag"])
-                parts.append({"ETag": etag, "PartNumber": part_num})
-                part_num += 1
+        part_num = 1  # AWS S3 needs part num to start with 1.
+        for block in tqdm(
+            data_batch_iterator, desc="Uploading to Gantry Bucket: ", total=total_num_parts
+        ):
+            # TODO: parallelize this client side?
+            response = requests.put(signed_urls[part_num - 1], data=block)
+            etag = str(response.headers["ETag"])
+            parts.append({"ETag": etag, "PartNumber": part_num})
+            part_num += 1
 
         logger.info("Starting Gantry Ingestion")
         # Submit file completion to gantry API.
@@ -290,12 +231,16 @@ class Gantry:
                 "upload_id": upload_id,
                 "key": file_key,
                 "parts": parts,
-                "data_link": gantry_data_link,
+                "data_link": asdict(data_link),
             },
             headers={"Content-Type": "application/json", "Accept": "application/json"},
         )
+        if "batch_id" not in complete_res:
+            raise GantryBatchCreationException(
+                "Failed to complete upload. Please contact Gantry support."
+            )
         batch_id = complete_res["batch_id"]
-        _batch_success_msg(batch_id, application, self.log_store)
+        _batch_success_msg(batch_id, data_link.application, self.log_store)
 
     @_log_exception
     @typechecked
@@ -304,13 +249,13 @@ class Gantry:
         application: str,
         version: Optional[Union[int, str]] = None,
         inputs: Optional[Union[List[dict], pd.DataFrame]] = None,
-        outputs: Optional[Union[List[dict], pd.DataFrame]] = None,
+        outputs: Optional[Union[List[Any], List[dict], pd.DataFrame]] = None,
         feedback_keys: Optional[List[str]] = None,
         feedback_ids: Optional[List[dict]] = None,
         feedbacks: Optional[Union[List[dict], pd.DataFrame]] = None,
         ignore_inputs: Optional[List[str]] = None,
         timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex]] = None,
-        sort_on_timestamp: bool = False,
+        sort_on_timestamp: bool = True,
         sample_rate: float = 1.0,
         as_batch: bool = False,
         tags: Optional[Dict[str, str]] = None,
@@ -360,7 +305,7 @@ class Gantry:
             feedback_ids (optional, List[dict]): A list of prediction feedback ids. The i-th entry
                 corresponds to the argument `feedback_ids` in
                 :meth:`gantry.client.Gantry.log_predictions` for the i-th prediction event.
-            feedbacks (Union[List[dict], pd.Dataframe]): A list of feedbacks. `feedbacks[i]`
+            feedbacks (Union[List[dict], pd.DataFrame]): A list of feedbacks. `feedbacks[i]`
                 is a dict of the features for the i-th prediction to be logged.
             ignore_inputs (optional, List[str]): A list of names of input features that should not
                 be monitored.
@@ -368,9 +313,9 @@ class Gantry:
                 prediction timestamps. If specified, `timestamps[i]` should be the timestamps
                 for the i-th prediction. If timestamps = None (default), then the prediction
                 timestamp defaults to the time when `log_records` is called.
-            sort_on_timestamp (bool, defaults to False): Works when timestamps are provided.
-                Sort using the given timestamp. Default to False.
-            sample_rate: Used for downsampling. The probability as a float that each record
+            sort_on_timestamp (bool, defaults to True): Works when timestamps are provided.
+                Sort using the given timestamp. Default to True.
+            sample_rate: Used for down-sampling. The probability as a float that each record
                 will be sent to Gantry.
             as_batch (bool, defaults to False): Whether to add batch metadata and tracking
                 in the 'batch' section of the dashboard
@@ -464,11 +409,11 @@ class Gantry:
         inputs: Union[List[dict], pd.DataFrame],
         outputs: Union[List[dict], pd.DataFrame],
         feedbacks: Union[List[dict], pd.DataFrame],
-        version: Optional[Union[int, str, List[Union[str, int]]]] = None,
+        version: Optional[Union[int, str]] = None,
         feedback_keys: Optional[List[str]] = None,
         ignore_inputs: Optional[List[str]] = None,
         timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex]] = None,
-        sort_on_timestamp: bool = False,
+        sort_on_timestamp: bool = True,
         sample_rate: float = 1.0,
         as_batch: bool = False,
         tags: Optional[Dict[str, str]] = None,
@@ -495,21 +440,10 @@ class Gantry:
                 )
             )
 
-        if version is not None:
-            if isinstance(version, (str, int)):
-                version = [version] * len(inputs)
-            elif len(version) != size:
-                # because of typing, this is a list
-                raise GantryLoggingException(
-                    "Size mismatch in version: {} data points, {} versions".format(
-                        size, len(version)
-                    )
-                )
-
         inputs = serializable_value(inputs)
         outputs = serializable_value(outputs)
         feedbacks = serializable_value(feedbacks)
-        serialized_version: List[Union[int, str]] = serializable_value(version)
+        version = serializable_value(version)
 
         environment, context = self._get_env_and_context(tags)
 
@@ -521,7 +455,7 @@ class Gantry:
             environment=environment,
             timestamp_idx=timestamp_idx,
             tags=tags,
-            version=serialized_version,
+            version=version,
             inputs=inputs,
             outputs=outputs,
             feedback_keys=feedback_keys,
@@ -532,18 +466,11 @@ class Gantry:
 
         if events:
             if as_batch:
-                batch = self._create_batch(
-                    events_len=len(events),
-                    type_=BatchTypes.RECORD.value,
-                    application=application,
+                self._upload_data_as_batch(
+                    application, environment, version, events, BatchType.RECORD
                 )
-                batch_id = batch["id"]
-                _enrich_events_with_batch_id(events, batch_id)
             else:
-                batch_id = None
-
-            self.log_store.log_batch(application, events)
-            _batch_success_msg(batch_id, application, self.log_store)
+                self.log_store.log_batch(application, events)
         else:
             logger.info("No events to log")
 
@@ -790,7 +717,7 @@ class Gantry:
         feedbacks: Union[List[dict], pd.DataFrame],
         feedback_version: Optional[Union[str, int]] = None,
         timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex]] = None,
-        sort_on_timestamp: bool = False,
+        sort_on_timestamp: bool = True,
         sample_rate: float = 1.0,
         as_batch: bool = False,
         tags: Optional[Dict[str, str]] = None,
@@ -827,12 +754,13 @@ class Gantry:
         try:
             feedback_ids = serializable_value(feedback_ids)
             feedbacks = serializable_value(feedbacks)
+            serialized_version: Union[str, int] = serializable_value(feedback_version)
 
             timestamp_idx = _create_timestamp_idx(sort_on_timestamp, timestamps, len(feedbacks))
 
             if len(feedback_ids) == len(feedbacks):
                 events = []
-                _, context = self._get_env_and_context(tags)
+                environment, context = self._get_env_and_context(tags)
 
                 for idx, timestamp in timestamp_idx:
                     try:
@@ -845,7 +773,7 @@ class Gantry:
                                 application,
                                 feedback_ids[idx],
                                 feedbacks[idx],
-                                feedback_version,
+                                serialized_version,
                                 timestamp,
                                 batch_id=batch_id,
                             )
@@ -863,19 +791,13 @@ class Gantry:
 
                 if events:
                     if as_batch:
-                        batch = self._create_batch(
-                            events_len=len(events),
-                            type_=BatchTypes.FEEDBACK.value,
-                            application=application,
+                        self._upload_data_as_batch(
+                            application, environment, serialized_version, events, BatchType.FEEDBACK
                         )
-                        batch_id = batch["id"]
-                        _enrich_events_with_batch_id(events, batch_id)
                     else:
                         batch_id = None
-
-                    self.log_store.log_batch(application, events)
-
-                    _batch_success_msg(batch_id, application, self.log_store)
+                        self.log_store.log_batch(application, events)
+                        _batch_success_msg(batch_id, application, self.log_store)
                 else:
                     logger.info("No events to log")
             else:
@@ -983,12 +905,12 @@ class Gantry:
         application: str,
         inputs: Union[List[dict], pd.DataFrame],
         outputs: Union[List[dict], List[Any], pd.DataFrame],
-        version: Optional[Union[int, str, List[Union[str, int]]]] = None,
+        version: Optional[Union[int, str]] = None,
         feedback_keys: Optional[List[str]] = None,
         ignore_inputs: Optional[List[str]] = None,
         feedback_ids: Optional[List[dict]] = None,
         timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex]] = None,
-        sort_on_timestamp: bool = False,
+        sort_on_timestamp: bool = True,
         sample_rate: float = 1.0,
         as_batch: bool = False,
         tags: Optional[Dict[str, str]] = None,
@@ -1023,10 +945,7 @@ class Gantry:
         try:
             inputs = serializable_value(inputs)
             outputs = serializable_value(outputs)
-            if version is not None and isinstance(version, (str, int)):
-                version = [version] * len(inputs)
             version = serializable_value(version) if version is not None else None
-            assert version is None or isinstance(version, list)  # mypy
 
             if len(inputs) == len(outputs):
                 if timestamps is not None and not len(timestamps) == len(inputs):
@@ -1067,18 +986,13 @@ class Gantry:
 
                 if events:
                     if as_batch:
-                        batch = self._create_batch(
-                            events_len=len(events),
-                            type_=BatchTypes.PREDICTION.value,
-                            application=application,
+                        self._upload_data_as_batch(
+                            application, environment, version, events, BatchType.PREDICTION
                         )
-                        batch_id = batch["id"]
-                        _enrich_events_with_batch_id(events, batch_id)
                     else:
                         batch_id = None
-
-                    self.log_store.log_batch(application, events)
-                    _batch_success_msg(batch_id, application, self.log_store)
+                        self.log_store.log_batch(application, events)
+                        _batch_success_msg(batch_id, application, self.log_store)
                 else:
                     logger.info("No events to log")
             else:
@@ -1088,27 +1002,31 @@ class Gantry:
 
             raise e
 
-    def _create_batch(self, events_len, type_, application, version=None):
-        batch_meta = _build_batch_meta(events_len, type_)
-
-        result = self.log_store._api_client.request(
-            "POST",
-            "/api/v1/applications/{}/batches".format(application),
-            json={"batch": batch_meta, "version": version},
+    def _upload_data_as_batch(
+        self,
+        application: str,
+        environment: str,
+        version: Optional[Union[str, int]],
+        events: List,
+        batch_type: BatchType,
+    ):
+        data_link: DataLink = _build_data_link(
+            application,
+            environment=environment,
+            version=version,
+            events=events,
+            batch_type=batch_type,
+        )
+        events_bytesize = sys.getsizeof(json.dumps(events, cls=EventEncoder).encode("utf-8"))
+        batch_count = int(
+            CHUNK_SIZE / (events_bytesize / data_link.num_events)
+        )  # max file chunk size / average event size -> count of batch events to get to chunk.
+        self._handle_upload(
+            _build_batch_iterator(events, batch_count),
+            data_link,
+            events_bytesize,
+            f"{application}_{uuid.uuid4()}",
         )
-
-        try:
-            check_response(result)
-        except GantryException:
-            # Raise a custom exception as this one raises
-            # to the main thread
-            raise GantryBatchCreationException(
-                "Failed to create batch: {} {}".format(result["response"], result.get("error"))
-            )
-
-        batch = result["data"]["batch"]
-
-        return batch
 
     @staticmethod
     def setup_logger(level="INFO"):
