@@ -3,6 +3,7 @@
 
 from gantry.query import distance, metric  # noqa: F401
 from gantry.query.main import (  # noqa: F401
+    create_view,
     init,
     list_application_environments,
     list_application_versions,
@@ -20,4 +21,5 @@ __all__ = [
     "list_application_versions",
     "list_application_views",
     "print_application_info",
+    "create_view",
 ]
