@@ -46,6 +46,7 @@ class GantryDataFrame(GantryQueryFrame):
         version: Optional[Union[int, str]] = None,
         env: Optional[str] = None,
         filters: Optional[List[dict]] = None,
+        tags: Optional[dict] = None,
     ):
         """
         :meta private: This is to skip constructor in the docs
@@ -59,8 +60,25 @@ class GantryDataFrame(GantryQueryFrame):
             QueryInfo(application_node_id, application, version, env, start_time, end_time),
         )
         self.filters = filters or []  # Holds raw dict filters to be used when fetching data
+        self.tags = tags  # Holds tags for filtering when fetching data
+
         self.series = self._build_series()
 
+    def create_view(self, name: str) -> None:
+        """Create a view from a Gantry Dataframe"""
+        # Inline import to avoid circular dependency.
+        from gantry.query.main import create_view
+
+        create_view(
+            name=name,
+            application=self.query_info.application,
+            start_time=self.query_info.start_time,
+            end_time=self.query_info.end_time,
+            version=self.query_info.version,
+            tag_filters=self.tags,
+            data_filters=self.filters,
+        )
+
     @classmethod
     def from_view(
         cls,
@@ -69,6 +87,7 @@ class GantryDataFrame(GantryQueryFrame):
         view: str,
         version: Optional[Union[str, int]],
         environment: Optional[str],
+        tags: Optional[dict] = None,
     ):
         version = version or get_last_application_version(
             api_client=api_client, application=application
@@ -94,6 +113,7 @@ class GantryDataFrame(GantryQueryFrame):
             version=version,
             env=view_obj["tag_filters"]["env"],
             filters=view_obj["data_filters"],
+            tags=tags,
         )
 
     def _clone_with_extra_filters(self, filters: "Filters") -> "GantryDataFrame":
@@ -125,6 +145,7 @@ class GantryDataFrame(GantryQueryFrame):
             start_time=self.query_info.start_time,
             end_time=self.query_info.end_time,
             filters=[*self.filters, *[f.filter_ for f in filters.filter_objs]],
+            tags=self.tags,
         )
 
     def __getitem__(self, key: str) -> Optional[Any]:
@@ -173,7 +194,7 @@ class GantryDataFrame(GantryQueryFrame):
         Returns:
             A pandas dataframe object.
         """
-        return self._head(n=n, query_filters=self.filters)
+        return self._head(n=n, query_filters=self.filters, tags=self.tags)
 
     def tail(self, n: int = 5) -> pd.DataFrame:  # type: ignore
         """Fetch the last n entries
@@ -184,7 +205,7 @@ class GantryDataFrame(GantryQueryFrame):
         Returns:
             A pandas dataframe object.
         """
-        return self._tail(n=n, query_filters=self.filters)
+        return self._tail(n=n, query_filters=self.filters, tags=self.tags)
 
     def fetch(self) -> pd.DataFrame:  # type: ignore
         """Fetch all entries
@@ -192,7 +213,7 @@ class GantryDataFrame(GantryQueryFrame):
         Returns:
             A pandas dataframe object.
         """
-        return self._fetch(query_filters=self.filters)
+        return self._fetch(query_filters=self.filters, tags=self.tags)
 
     def _aggregation_factory(self, aggregration: str):
         """Generates methods that calculate aggregations on the series
@@ -496,7 +517,12 @@ class GantrySeries(GantryQueryFrame):
         Returns:
             A pandas dataframe object.
         """
-        return self._head(n=n, column=self.name, query_filters=self.parent_dataframe.filters)
+        return self._head(
+            n=n,
+            column=self.name,
+            query_filters=self.parent_dataframe.filters,
+            tags=self.parent_dataframe.tags,
+        )
 
     def tail(self, n: int = 5) -> pd.DataFrame:  # type: ignore
         """Fetch the last n entries
@@ -507,7 +533,12 @@ class GantrySeries(GantryQueryFrame):
         Returns:
             A pandas dataframe object.
         """
-        return self._tail(n=n, column=self.name, query_filters=self.parent_dataframe.filters)
+        return self._tail(
+            n=n,
+            column=self.name,
+            query_filters=self.parent_dataframe.filters,
+            tags=self.parent_dataframe.tags,
+        )
 
     def fetch(self) -> pd.DataFrame:  # type: ignore
         """Fetch all entries
@@ -515,7 +546,11 @@ class GantrySeries(GantryQueryFrame):
         Returns:
             A pandas dataframe object.
         """
-        return self._fetch(column=self.name, query_filters=self.parent_dataframe.filters)
+        return self._fetch(
+            column=self.name,
+            query_filters=self.parent_dataframe.filters,
+            tags=self.parent_dataframe.tags,
+        )
 
     @runs_on("int", "float")
     def mean(self) -> float:
@@ -661,6 +696,7 @@ class GantrySeries(GantryQueryFrame):
                 "stat_kwargs": {attribute: attribute_kwargs},
                 "feature": self.name,
                 "filters": self.parent_dataframe.filters,
+                "tags": self.parent_dataframe.tags,
             }
         }
         response = _raw_query_cached_request(
