@@ -1,7 +1,6 @@
 import datetime
 import json
 import logging
-from distutils.util import strtobool
 
 import mock
 import numpy as np
@@ -10,10 +9,11 @@ import pytest
 import responses
 from freezegun import freeze_time
 
-from gantry.config import Config
 from gantry.exceptions import GantryBatchCreationException, GantryLoggingDataTypeError
 from gantry.logger import client
+from gantry.logger.constants import BatchType, UploadFileType
 from gantry.logger.stores import BaseLogStore
+from gantry.logger.types import DataLink, DataLinkElement
 
 CURRENT_TIME, CURRENT_TIME_STR = datetime.datetime(2032, 5, 23, 0, 0, 0), "2032-05-23T00:00:00"
 FUTURE_TIME = datetime.datetime(2042, 5, 23, 0, 0, 0)
@@ -170,34 +170,6 @@ def test_build_prediction_event(mock_uuid, timestamp, feedback_keys, feedback_id
         )
 
 
-@pytest.mark.parametrize(
-    "logs_location",
-    ["kinesis://foobar", "/tmp/foo/bar", "psql://some-database", "app.staging.gantry.io", None, ""],
-)
-def test_from_config_constructor_invalid(logs_location):
-    config = Config(logs_location=logs_location)
-    with pytest.raises(ValueError):
-        client.Gantry.from_config(config)
-
-
-@pytest.mark.parametrize("bypass_firehose", ["true", "false"])
-@pytest.mark.parametrize("api_key", ["this-is-an-api-key", None])
-@pytest.mark.parametrize("logs_location", ["https://foo.bar.io", "http://foo.bar.io"])
-@pytest.mark.parametrize("send_in_background", ["true", "false"])
-def test_from_config_constructor(logs_location, api_key, bypass_firehose, send_in_background):
-    config = Config(
-        logs_location=logs_location,
-        api_key=api_key,
-        bypass_firehose=bypass_firehose,
-        send_in_background=send_in_background,
-    )
-    cli = client.Gantry.from_config(config)
-    assert cli.log_store._bypass_firehose is bool(strtobool(bypass_firehose))
-    assert cli.log_store._api_client._host == logs_location
-    assert cli.log_store._api_client._api_key == api_key
-    assert cli.log_store.send_in_background is bool(strtobool(send_in_background))
-
-
 def test_ping_fail(cli_obj):
     cli_obj.log_store.status = False
     assert not cli_obj.ping()
@@ -1188,7 +1160,7 @@ def test_log_feedback_mismatch_size(cli_obj):
         feedbacks=[{100: "some-value"}, {200: "some-other-value"}],
         feedback_version=10,
         timestamps=[ANOTHER_TIME, SOME_TIME],
-        sort_on_timestamp=False,
+        sort_on_timestamp=True,
         as_batch=False,
     )
     assert cli_obj.log_store.logs == []
@@ -1624,95 +1596,183 @@ def test_batch_id_population_without_batch(initialized_gantry, tmp_path):
             assert logged_ev["batch_id"] is None
 
 
-def test_batch_id_population_with_batch_pred(init_gantry_with_api):
+@freeze_time(CURRENT_TIME)
+@pytest.mark.parametrize("version", ["0", 0, None])
+@pytest.mark.parametrize("sort", [True, False])
+@pytest.mark.parametrize("method", ["log_predictions", "log_records_prediction", "log_records"])
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
+@mock.patch("gantry.logger.client._batch_success_msg")
+def test_log_predictions_as_batch(
+    mock_batch_success, mock_uuid, version, sort, method, init_gantry_with_api
+):
+    if method == "log_records":
+        size = 40
+    else:
+        size = 20
+    mock_uuid.side_effect = [str(x) for x in range(size + 1)]
+
+    inputs = [{"input1": i, "input2": 2} for i in range(20)]
+    outputs = [i % 2 for i in range(20)]
+    feedbacks = [{"val": i} for i in range(20)]
+
+    with responses.RequestsMock() as resp:
+        resp.add(
+            resp.GET,
+            f"{HOST}/api/v1/ingest/file-preupload?filename=simple_test_{size}&num_parts=1&filetype=EVENTS_BYTES",  # noqa
+            json={
+                "response": "ok",
+                "upload_urls": ["http://test_url"],
+                "upload_id": "10",
+                "key": "x1234",
+            },
+        )
+        resp.add(
+            resp.PUT,
+            "http://test_url",
+            headers={"Content-Type": "application/json", "ETag": "working_etag"},
+        )
+        resp.add(
+            resp.POST,
+            url=f"{HOST}/api/v1/ingest/file",
+            json={"response": "ok", "batch_id": "hello_123"},
+            headers={"Content-Type": "application/json"},
+        )
+
+        if method == "log_predictions":
+            init_gantry_with_api.log_predictions(
+                "simple_test", inputs, outputs, version, sort_on_timestamp=sort, as_batch=True
+            )
+        elif method == "log_records_prediction":
+            init_gantry_with_api.log_records(
+                "simple_test",
+                version,
+                inputs=inputs,
+                outputs=outputs,
+                sort_on_timestamp=sort,
+                as_batch=True,
+            )
+        elif method == "log_records":
+            init_gantry_with_api.log_records(
+                "simple_test",
+                version,
+                inputs=inputs,
+                outputs=outputs,
+                feedbacks=feedbacks,
+                sort_on_timestamp=sort,
+                as_batch=True,
+            )
+        mock_batch_success.assert_called_once_with(
+            "hello_123", "simple_test", init_gantry_with_api.get_client().log_store
+        )
+
+
+@pytest.mark.parametrize("version", ["0", 0, None])
+@pytest.mark.parametrize("sort", [True, False])
+@pytest.mark.parametrize("method", ["log_feedback", "log_records_feedback"])
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
+@mock.patch("gantry.logger.client._batch_success_msg")
+def test_log_feedback_as_batch(
+    mock_batch_success, mock_uuid, version, sort, method, init_gantry_with_api
+):
+    size = 2
+    mock_uuid.side_effect = [str(x) for x in range(size + 1)]
+    feedbacks = [{"value": "foo"} for _ in range(size)]
+
+    with responses.RequestsMock() as resp:
+        resp.add(
+            resp.GET,
+            f"{HOST}/api/v1/ingest/file-preupload?filename=simple_test_{size}&num_parts=1&filetype=EVENTS_BYTES",  # noqa
+            json={
+                "response": "ok",
+                "upload_urls": ["http://test_url"],
+                "upload_id": "10",
+                "key": "x1234",
+            },
+        )
+        resp.add(
+            resp.PUT,
+            "http://test_url",
+            headers={"Content-Type": "application/json", "ETag": "working_etag"},
+        )
+        resp.add(
+            resp.POST,
+            f"{HOST}/api/v1/ingest/file",
+            json={"response": "ok", "batch_id": "hello_123"},
+            headers={"Content-Type": "application/json"},
+        )
+
+        if method == "log_feedback":
+            init_gantry_with_api.log_feedback(
+                application="simple_test",
+                feedback_ids=feedbacks,
+                feedbacks=feedbacks,
+                feedback_version=version,
+                sort_on_timestamp=sort,
+                as_batch=True,
+            )
+        elif method == "log_records_feedback":
+            init_gantry_with_api.log_records(
+                "simple_test",
+                version,
+                feedback_ids=feedbacks,
+                feedbacks=feedbacks,
+                sort_on_timestamp=sort,
+                as_batch=True,
+            )
+        mock_batch_success.assert_called_once_with(
+            "hello_123", "simple_test", init_gantry_with_api.get_client().log_store
+        )
+
+
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
+def test_batch_completion_fail(mock_uuid, init_gantry_with_api):
     size = 20
     inputs = [{"input1": i, "input2": 2} for i in range(size)]
     outputs = [i % 2 for i in range(size)]
 
+    mock_uuid.side_effect = [str(x) for x in range(size + 1)]
+
     with responses.RequestsMock() as resp:
         resp.add(
-            resp.POST,
-            "{}/api/v1/applications/simple_test/batches".format(HOST),
+            resp.GET,
+            f"{HOST}/api/v1/ingest/file-preupload?filename=simple_test_{size}&num_parts=1&filetype=EVENTS_BYTES",  # noqa
             json={
                 "response": "ok",
-                "data": {
-                    "batch": {
-                        "id": "9f204f7f-37e0-42b1-ab6b-748b1e47566e",
-                        "type": "PREDICTION",
-                        "size": 100,
-                        "failed_at": None,
-                        "created_at": "2022-02-28 18:56:25.814004",
-                        "updated_at": "2022-02-28 18:56:25.814009",
-                        "finished_at": None,
-                        "started_at": "2022-02-28 18:56:25.796876",
-                        "application_node_id": "ae6c37f4-8a66-46a3-aa5a-a7661ec18a11",
-                    }
-                },
+                "upload_urls": ["http://test_url"],
+                "upload_id": "10",
+                "key": "x1234",
             },
         )
         resp.add(
-            resp.POST,
-            "{}/api/v1/ingest/raw".format(HOST),
-            json={"response": "ok"},
-            headers={"Content-Type": "application/json"},
-        )
-
-        init_gantry_with_api.log_predictions(
-            "simple_test", inputs, outputs, sort_on_timestamp=True, as_batch=True
-        )
-
-
-def test_batch_id_population_with_batch_feedback(init_gantry_with_api):
-    with responses.RequestsMock() as resp:
-        resp.add(
-            resp.POST,
-            "{}/api/v1/applications/simple_test/batches".format(HOST),
-            json={
-                "response": "ok",
-                "data": {
-                    "batch": {
-                        "id": "9f204f7f-37e0-42b1-ab6b-748b1e47566e",
-                        "type": "PREDICTION",
-                        "size": 100,
-                        "failed_at": None,
-                        "created_at": "2022-02-28 18:56:25.814004",
-                        "updated_at": "2022-02-28 18:56:25.814009",
-                        "finished_at": None,
-                        "started_at": "2022-02-28 18:56:25.796876",
-                        "application_node_id": "ae6c37f4-8a66-46a3-aa5a-a7661ec18a11",
-                    }
-                },
-            },
+            resp.PUT,
+            "http://test_url",
+            headers={"Content-Type": "application/json", "ETag": "working_etag"},
         )
         resp.add(
             resp.POST,
-            "{}/api/v1/ingest/raw".format(HOST),
-            json={"response": "ok"},
+            f"{HOST}/api/v1/ingest/file",
+            json={"response": "not-ok"},
             headers={"Content-Type": "application/json"},
         )
 
-        init_gantry_with_api.log_feedback(
-            application="simple_test",
-            feedback_ids=[{"value": "bar"}],
-            feedbacks=[{"value": "foo"}],
-            feedback_version=10,
-            as_batch=True,
-        )
+        with pytest.raises(GantryBatchCreationException):
+            init_gantry_with_api.log_predictions(
+                "simple_test", inputs, outputs, sort_on_timestamp=True, as_batch=True
+            )
 
 
-def test_batch_fail(init_gantry_with_api):
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
+def test_batch_preupload_fail(mock_uuid, init_gantry_with_api):
     size = 20
     inputs = [{"input1": i, "input2": 2} for i in range(size)]
     outputs = [i % 2 for i in range(size)]
+    mock_uuid.side_effect = [str(x) for x in range(size + 1)]
 
     with responses.RequestsMock() as resp:
         resp.add(
-            resp.POST,
-            "{}/api/v1/applications/simple_test/batches".format(HOST),
-            json={
-                "response": "not-ok",
-                "data": {},
-            },
+            resp.GET,
+            f"{HOST}/api/v1/ingest/file-preupload?filename=simple_test_{size}&num_parts=1&filetype=EVENTS_BYTES",  # noqa
+            json={"response": "broken-return"},
         )
 
         with pytest.raises(GantryBatchCreationException):
@@ -1792,32 +1852,13 @@ def test_log_predictions_and_feedbacks_size_mismatch_timestamps(timestamps, cli_
     assert log_store.logs == []
 
 
-@pytest.mark.parametrize(
-    "version",
-    [
-        [],
-        ["2.0", "2.1"],
-    ],
-)
-def test_log_predictions_and_feedbacks_size_mismatch_versions(version, cli_obj, log_store):
-    cli_obj._log_predictions_and_feedbacks(
-        application="foobar",
-        inputs=[{"A": 100}],
-        outputs=[{"B": 200}],
-        feedback_keys=["A"],
-        feedbacks=[{"A": 100}],
-        version=version,
-    )
-
-    assert log_store.logs == []
-
-
 @freeze_time(CURRENT_TIME)
 @pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
 @pytest.mark.parametrize(
     "timestamps", [None, pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME]]
 )
 @pytest.mark.parametrize("sort", [False, True])
+@pytest.mark.parametrize("version", ["0", 0, None])
 @pytest.mark.parametrize("as_batch", [False, True])
 @pytest.mark.parametrize(
     "test_feedbacks",
@@ -1841,25 +1882,26 @@ def test_log_predictions_and_feedbacks_size_mismatch_versions(version, cli_obj,
     ],
 )
 @mock.patch("gantry.logger.event_builder.uuid.uuid4")
-@mock.patch("gantry.logger.client.Gantry._create_batch")
+@mock.patch("gantry.logger.client.Gantry._upload_data_as_batch")
 def test_log_predictions_and_feedbacks_df(
-    mock_create_batch,
+    mock_upload_data_as_batch,
     mock_uuid,
     test_inputs,
     test_outputs,
     test_feedbacks,
     as_batch,
+    version,
     sort,
     timestamps,
     tags,
     cli_obj,
     log_store,
 ):
-    mock_create_batch.return_value = {"id": "batch-id-123"}
+    mock_upload_data_as_batch.return_value = None
     mock_uuid.side_effect = ["ABCD1", "ABCD2", "ABCD3", "ABCD4"]
     cli_obj._log_predictions_and_feedbacks(
         application="foobar",
-        version="2.0.1",
+        version=version,
         inputs=test_inputs,
         outputs=test_outputs,
         feedback_keys=["A"],
@@ -1872,64 +1914,151 @@ def test_log_predictions_and_feedbacks_df(
     expected_tags = tags if tags else {"env": "test"}
 
     event_1 = {
-        "batch_id": "batch-id-123" if as_batch else None,
         "event_id": "ABCD2" if not (sort and timestamps is not None) else "ABCD4",
-        "feedback": {"A": 200},
-        "feedback_id": "5dd14615efeb2d086e519ed35efd3f73",
-        "feedback_id_inputs": {"A": 100},
-        "inputs": {"A": 100},
         "log_timestamp": CURRENT_TIME_STR,
+        "timestamp": ANOTHER_TIME_STR if timestamps is not None else CURRENT_TIME_STR,
         "metadata": {
             "environment": tags["env"] if tags else "test",
-            "feedback_keys": ["A"],
-            "feedback_version": "2.0.1",
             "func_name": "foobar",
+            "version": version,
+            "feedback_keys": ["A"],
             "ignore_inputs": None,
             "provided_feedback_id": None,
-            "version": "2.0.1",
+            "feedback_version": version,
         },
+        "inputs": {"A": 100},
         "outputs": {"B": 300},
+        "feedback_id": "5dd14615efeb2d086e519ed35efd3f73",
         "tags": expected_tags,
-        "timestamp": ANOTHER_TIME_STR if timestamps is not None else CURRENT_TIME_STR,
+        "batch_id": None,
+        "feedback_id_inputs": {"A": 100},
+        "feedback": {"A": 200},
     }
 
     event_2 = {
-        "batch_id": "batch-id-123" if as_batch else None,
         "event_id": "ABCD4" if not (sort and timestamps is not None) else "ABCD2",
-        "feedback": {"A": 201},
-        "feedback_id": "9e329293e022d6cdaafdec49b5f4fedc",
-        "feedback_id_inputs": {"A": 101},
-        "inputs": {"A": 101},
         "log_timestamp": CURRENT_TIME_STR,
+        "timestamp": SOME_TIME_STR if timestamps is not None else CURRENT_TIME_STR,
         "metadata": {
             "environment": tags["env"] if tags else "test",
-            "feedback_keys": ["A"],
-            "feedback_version": "2.0.1",
             "func_name": "foobar",
+            "version": version,
+            "feedback_keys": ["A"],
             "ignore_inputs": None,
             "provided_feedback_id": None,
-            "version": "2.0.1",
+            "feedback_version": version,
         },
+        "inputs": {"A": 101},
         "outputs": {"B": 301},
+        "feedback_id": "9e329293e022d6cdaafdec49b5f4fedc",
         "tags": expected_tags,
-        "timestamp": SOME_TIME_STR if timestamps is not None else CURRENT_TIME_STR,
+        "batch_id": None,
+        "feedback_id_inputs": {"A": 101},
+        "feedback": {"A": 201},
     }
 
-    assert log_store.logs == [
-        {
-            "events": [event_1, event_2]
-            if not (sort and timestamps is not None)
-            else [event_2, event_1],
-            "application": "foobar",
-        },
-    ]
-
     if as_batch:
-        mock_create_batch.assert_called_once_with(
-            events_len=2, type_="RECORD", application="foobar"
+        mock_upload_data_as_batch.assert_called_once_with(
+            "foobar",
+            tags["env"] if tags else "test",
+            version,
+            [event_1, event_2] if not (sort and timestamps is not None) else [event_2, event_1],
+            BatchType.RECORD,
         )
     else:
-        mock_create_batch.assert_not_called()
+        assert log_store.logs == [
+            {
+                "events": [event_1, event_2]
+                if not (sort and timestamps is not None)
+                else [event_2, event_1],
+                "application": "foobar",
+            },
+        ]
+        mock_upload_data_as_batch.assert_not_called()
+
+
+@freeze_time(CURRENT_TIME)
+@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
+@pytest.mark.parametrize(
+    "timestamps", [None, pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME]]
+)
+@pytest.mark.parametrize("sort", [False, True])
+@pytest.mark.parametrize("version", ["0", 0, None])
+@mock.patch("gantry.logger.event_builder.uuid.uuid4")
+@mock.patch("gantry.logger.client.Gantry._handle_upload")
+def test__upload_data_as_batch(
+    mock_handle_upload,
+    mock_uuid,
+    version,
+    sort,
+    timestamps,
+    tags,
+    cli_obj,
+):
+    mock_uuid.side_effect = ["1"]
+    expected_tags = tags if tags else {"env": "test"}
+    event_1 = {
+        "event_id": "ABCD2" if not (sort and timestamps is not None) else "ABCD4",
+        "log_timestamp": CURRENT_TIME_STR,
+        "timestamp": ANOTHER_TIME_STR if timestamps is not None else CURRENT_TIME_STR,
+        "metadata": {
+            "environment": tags["env"] if tags else "test",
+            "func_name": "foobar",
+            "version": version,
+            "feedback_keys": ["A"],
+            "ignore_inputs": None,
+            "provided_feedback_id": None,
+            "feedback_version": version,
+        },
+        "inputs": {"A": 100},
+        "outputs": {"B": 300},
+        "feedback_id": "5dd14615efeb2d086e519ed35efd3f73",
+        "tags": expected_tags,
+        "batch_id": None,
+        "feedback_id_inputs": {"A": 100},
+        "feedback": {"A": 200},
+    }
+    event_2 = {
+        "event_id": "ABCD4" if not (sort and timestamps is not None) else "ABCD2",
+        "log_timestamp": CURRENT_TIME_STR,
+        "timestamp": SOME_TIME_STR if timestamps is not None else CURRENT_TIME_STR,
+        "metadata": {
+            "environment": tags["env"] if tags else "test",
+            "func_name": "foobar",
+            "version": version,
+            "feedback_keys": ["A"],
+            "ignore_inputs": None,
+            "provided_feedback_id": None,
+            "feedback_version": version,
+        },
+        "inputs": {"A": 101},
+        "outputs": {"B": 301},
+        "feedback_id": "9e329293e022d6cdaafdec49b5f4fedc",
+        "tags": expected_tags,
+        "batch_id": None,
+        "feedback_id_inputs": {"A": 101},
+        "feedback": {"A": 201},
+    }
+    events = [event_1, event_2]
+
+    cli_obj._upload_data_as_batch(
+        "test_app", tags["env"] if tags else "test", version, events, BatchType.RECORD
+    )
+
+    mock_handle_upload.assert_called_once_with(
+        mock.ANY,
+        DataLink(
+            application="test_app",
+            version=str(version) if version is not None else version,
+            file_type=UploadFileType.EVENTS_BYTES,
+            batch_type=BatchType.RECORD,
+            log_timestamp=CURRENT_TIME.isoformat(),
+            num_events=2,
+            tags={"env": DataLinkElement(val=tags["env"] if tags else "test")},
+        ),
+        mock.ANY,
+        "test_app_1",
+    )
 
 
 @pytest.mark.parametrize(
@@ -1975,7 +2104,7 @@ def test_log_records_preds_and_feedback(
         feedback_keys=["A"],
         feedbacks=test_feedbacks,
         timestamps=None,
-        sort_on_timestamp=False,
+        sort_on_timestamp=True,
     )
 
     mock_preds_and_feedback.assert_called_once()
@@ -2018,7 +2147,7 @@ def test_log_records_preds_and_feedback_0_5(version, test_inputs, test_outputs,
             feedback_keys=["A"],
             feedbacks=test_feedbacks,
             timestamps=None,
-            sort_on_timestamp=False,
+            sort_on_timestamp=True,
             sample_rate=0.5,
             as_batch=False,
         )
@@ -2063,7 +2192,7 @@ def test_log_records_preds_and_feedback_0_5_none(
             feedback_keys=["A"],
             feedbacks=test_feedbacks,
             timestamps=None,
-            sort_on_timestamp=False,
+            sort_on_timestamp=True,
             sample_rate=0.5,
             as_batch=False,
         )
@@ -2106,7 +2235,7 @@ def test_log_records_preds_and_feedback_0_0(version, test_inputs, test_outputs,
             feedback_keys=["A"],
             feedbacks=test_feedbacks,
             timestamps=None,
-            sort_on_timestamp=False,
+            sort_on_timestamp=True,
             sample_rate=0.0,
             as_batch=False,
         )
@@ -2143,7 +2272,7 @@ def test_log_records_preds_only(
         feedback_keys=["A"],
         feedbacks=None,
         timestamps=None,
-        sort_on_timestamp=False,
+        sort_on_timestamp=True,
     )
 
     mock_preds_and_feedback.assert_not_called()
@@ -2173,7 +2302,7 @@ def test_log_records_feedback_only(
         feedback_keys=["A"],
         feedbacks=test_feedbacks,
         timestamps=None,
-        sort_on_timestamp=False,
+        sort_on_timestamp=True,
     )
 
     mock_preds_and_feedback.assert_not_called()
@@ -2215,7 +2344,7 @@ def test_log_records_no_data(
         feedback_keys=["A"],
         feedbacks=test_feedbacks,
         timestamps=None,
-        sort_on_timestamp=False,
+        sort_on_timestamp=True,
     )
 
     mock_preds_and_feedback.assert_not_called()
@@ -2245,7 +2374,7 @@ def test_log_records_incomplete_preds(
         feedback_keys=["A"],
         feedbacks=test_feedbacks,
         timestamps=None,
-        sort_on_timestamp=False,
+        sort_on_timestamp=True,
     )
 
     mock_preds_and_feedback.assert_not_called()
