@@ -1,13 +1,16 @@
+import datetime
+
 import mock
 import pytest
 import responses
 from responses import matchers
 
 from gantry.api_client import APIClient
+from gantry.exceptions import GantryException
 from gantry.query.client import GantryQuery
 from gantry.query.core.dataframe import GantryDataFrame
 
-from .conftest import END_TIME, ORIGIN, START_TIME
+from .conftest import END_TIME, ORIGIN, SOME_DURATION, START_TIME
 
 
 @pytest.fixture
@@ -131,11 +134,13 @@ def test_query_with_view(mock_application_views, mock_factory, gantry_query_obj)
     mock_factory.return_value = "data"
 
     assert (
-        gantry_query_obj.query("foobar", version="1.2.3", environment="foo", view="barbaz")
+        gantry_query_obj.query(
+            "foobar", version="1.2.3", environment="foo", view="barbaz", tags={"foo": "bar"}
+        )
         == "data"
     )
     mock_factory.assert_called_once_with(
-        gantry_query_obj._api_client, "foobar", "barbaz", "1.2.3", "foo"
+        gantry_query_obj._api_client, "foobar", "barbaz", "1.2.3", "foo", tags={"foo": "bar"}
     )
 
 
@@ -197,6 +202,155 @@ def test_add_metric(gantry_query_obj):
         gantry_query_obj.add_metric("application", {})
 
 
+@pytest.mark.parametrize(
+    ["duration", "start_time", "end_time"],
+    [
+        (SOME_DURATION, START_TIME, None),
+        (SOME_DURATION, None, END_TIME),
+        (SOME_DURATION, START_TIME, END_TIME),
+        (None, None, END_TIME),
+        (None, START_TIME, None),
+        (None, None, None),
+    ],
+)
+@mock.patch("gantry.query.client.get_application_node_id")
+def test_create_view_invalid_params(
+    mock_get_app_node_id, gantry_query_obj, duration, start_time, end_time
+):
+    mock_get_app_node_id.return_value = "foobar12345"
+    with pytest.raises(ValueError):
+        gantry_query_obj.create_view(
+            application="foo",
+            name="bar",
+            version="2",
+            duration=duration,
+            start_time=start_time,
+            end_time=end_time,
+        )
+
+
+@pytest.mark.parametrize(
+    ["kwargs", "expected_data"],
+    [
+        (
+            {
+                "application": "baz",
+                "name": "foo",
+                "version": "1.2.3",
+                "tag_filters": {"foo": "bar"},
+                "data_filters": [{"some": "filter"}],
+                "duration": datetime.timedelta(days=1),
+            },
+            {
+                "name": "foo",
+                "model_node_id": "foobar12345",
+                "tag_filters": {"foo": "bar"},
+                "data_filters": [{"some": "filter"}],
+                "details": {"color": "#FF8900"},
+                "duration": "P1D",
+            },
+        ),
+        (
+            {
+                "application": "baz",
+                "name": "foo",
+                "version": "1.2.3",
+                "data_filters": [],
+                "start_time": START_TIME,
+                "end_time": END_TIME,
+            },
+            {
+                "name": "foo",
+                "model_node_id": "foobar12345",
+                "tag_filters": {},
+                "data_filters": [],
+                "details": {"color": "#FF8900"},
+                "start_time": "2008-09-03T20:56:35Z",
+                "end_time": "2008-09-10T23:58:23Z",
+            },
+        ),
+    ],
+)
+@mock.patch("gantry.query.client.get_application_node_id")
+def test_create_view(mock_get_app_node_id, kwargs, expected_data, gantry_query_obj):
+    mock_get_app_node_id.return_value = "foobar12345"
+    with responses.RequestsMock() as rsps:
+        rsps.add(
+            responses.POST,
+            "{}/api/v1/views".format(ORIGIN),
+            json={
+                "response": "ok",
+                "data": "OK",
+            },
+            match=[matchers.json_params_matcher(expected_data)],
+        )
+
+        gantry_query_obj.create_view(**kwargs)
+    mock_get_app_node_id.assert_called_once_with(
+        gantry_query_obj._api_client, kwargs["application"], version=kwargs["version"]
+    )
+
+
+@pytest.mark.parametrize(
+    ["kwargs", "expected_data"],
+    [
+        (
+            {
+                "application": "baz",
+                "name": "foo",
+                "version": "1.2.3",
+                "tag_filters": {"foo": "bar"},
+                "data_filters": [{"some": "filter"}],
+                "duration": datetime.timedelta(days=1),
+            },
+            {
+                "name": "foo",
+                "model_node_id": "foobar12345",
+                "tag_filters": {"foo": "bar"},
+                "data_filters": [{"some": "filter"}],
+                "details": {"color": "#FF8900"},
+                "duration": "P1D",
+            },
+        ),
+        (
+            {
+                "application": "baz",
+                "name": "foo",
+                "version": "1.2.3",
+                "data_filters": [],
+                "start_time": START_TIME,
+                "end_time": END_TIME,
+            },
+            {
+                "name": "foo",
+                "model_node_id": "foobar12345",
+                "tag_filters": {},
+                "data_filters": [],
+                "details": {"color": "#FF8900"},
+                "start_time": "2008-09-03T20:56:35Z",
+                "end_time": "2008-09-10T23:58:23Z",
+            },
+        ),
+    ],
+)
+@mock.patch("gantry.query.client.get_application_node_id")
+def test_create_view_error(mock_get_app_node_id, kwargs, expected_data, gantry_query_obj):
+    mock_get_app_node_id.return_value = "foobar12345"
+    with responses.RequestsMock() as rsps:
+        rsps.add(
+            responses.POST,
+            "{}/api/v1/views".format(ORIGIN),
+            json={
+                "response": "not ok",
+                "data": "OK",
+            },
+            match=[matchers.json_params_matcher(expected_data)],
+        )
+
+        with pytest.raises(GantryException):
+            gantry_query_obj.create_view(**kwargs)
+
+
 VIEWS_DATA = [
     {"name": "view_1", "tag_filters": {"env": "env-A"}, "other": "A"},
     {"name": "view_2", "tag_filters": {"env": "env-B"}, "other": "B"},
