@@ -64,13 +64,26 @@ def test_global_init(mock_globals):
             "query",
             {
                 "application": "foobar",
-                "version": "1.2.3",
                 "start_time": "2020",
                 "end_time": "2020",
                 "version": "1.2.3",
                 "environment": "dev",
                 "filters": [],
                 "view": "foo",
+                "tags": {"foo": "bar"},
+            },
+        ),
+        (
+            "create_view",
+            {
+                "name": "barbaz",
+                "application": "foobar",
+                "version": "1.2.3",
+                "start_time": "2020",
+                "end_time": "2020",
+                "duration": "1D",
+                "data_filters": [],
+                "tag_filters": {"foo": "bar"},
             },
         ),
         (
